@@ -600,6 +600,7 @@ impl Add {
 
     pub fn to_counts(&self) -> HashMap<CircuitRc, usize> {
         let mut counts = HashMap::default();
+        counts.reserve(20);
         for item in &self.nodes {
             *counts.entry(item.clone()).or_insert(0) += 1;
         }
@@ -1285,6 +1286,26 @@ impl Index {
             name,
         )
     }
+
+    pub fn child_axis_map_including_slices(&self) -> Vec<Vec<Option<usize>>> {
+        let mut cur: i32 = -1;
+        let result = vec![self
+            .index
+            .0
+            .iter()
+            .map(|idx| {
+                if !matches!(idx, TensorAxisIndex::Single(_)) {
+                    cur += 1;
+                }
+                if matches!(idx, TensorAxisIndex::Slice(_)) {
+                    Some(cur as usize)
+                } else {
+                    None
+                }
+            })
+            .collect()];
+        result
+    }
 }
 
 #[pyclass(extends=PyCircuitBase)]
