@@ -213,8 +213,9 @@ impl Parser {
         let mut partial_circuits: HashMap<CircuitIdent, PartialCirc> = HashMap::default();
         // ident and if it's a new node
         let mut stack: Vec<(CircuitIdent, bool)> = vec![];
+        let mut was_previous_only_child_marker = false;
         let mut top_level = vec![];
-        const WHITESPACE: &str = r"[\s│└├‣]*";
+        const WHITESPACE: &str = r"[\s│└├‣─]*";
         const NAME_INSIDE_MATCH: &str = r"(?:(?:\\')?[^']*)*";
 
         let (
@@ -249,12 +250,20 @@ impl Parser {
             // supports newlines and comment lines starting with #
             Regex::new(&format!(r"^{ws}(#.*)?$", ws = WHITESPACE)).unwrap()
         });
+        static RE_ONLY_CHILD_MARKER: Lazy<Regex> = Lazy::new(|| {
+            // supports newlines and comment lines starting with #
+            Regex::new(&format!(r"^{ws}(?:▼|\|/)(#.*)?$", ws = WHITESPACE)).unwrap()
+        });
         let mut first_num_spaces = None;
         let mut first_line = None;
         for line in lines {
             if RE_SKIP_LINE.is_match(line) {
                 continue;
             }
+            if RE_ONLY_CHILD_MARKER.is_match(line) {
+                was_previous_only_child_marker = true;
+                continue;
+            }
 
             let re_captures =
                 re.captures(line)
@@ -291,7 +300,8 @@ impl Parser {
                     line: get_unindented_line(),
                 });
             }
-            let indentation_level = num_spaces / tab_width;
+            let indentation_level =
+                num_spaces / tab_width + (was_previous_only_child_marker as usize);
             if indentation_level > stack.len() {
                 bail!(ParseCircuitError::InvalidIndentation {
                     tab_width,
@@ -460,6 +470,10 @@ impl Parser {
                     .children
                     .push(circuit_ident.clone());
             }
+            if was_previous_only_child_marker {
+                stack.pop();
+                was_previous_only_child_marker = false;
+            }
             stack.push((circuit_ident, is_new_node));
         }
 
