@@ -41,6 +41,7 @@ const TEE: &'static str = "├";
 const ARROW: &'static str = "‣";
 const UP_ELBOW: &'static str = "└";
 const DOWN_ELBOW: &'static str = "┌";
+const HORIZ_BAR: &'static str = "─";
 
 fn_struct!(pub CircuitToColorCode:Fn(circuit:CircuitRc)->Option<usize>);
 fn_struct!(pub CircuitCommenter:Fn(circuit:CircuitRc)->String);
@@ -70,6 +71,9 @@ pub struct PrintOptions {
     #[pyo3(get, set)]
     pub sync_tensors: bool,
     #[pyo3(get, set)]
+    pub seen_children_same_line: bool,
+    #[pyo3(get, set)]
+    pub only_child_below: bool,
     pub tensor_index_literal: bool,
 }
 
@@ -87,6 +91,8 @@ impl Default for PrintOptions {
             comment_arg_names: false,
             commenters: vec![],
             sync_tensors: false,
+            seen_children_same_line: false,
+            only_child_below: false,
             tensor_index_literal: false,
         }
         .validate_ret()
@@ -189,6 +195,8 @@ impl PrintOptions {
         comment_arg_names = "PrintOptions::default().comment_arg_names",
         commenters = "PrintOptions::default().commenters",
         sync_tensors = "PrintOptions::default().sync_tensors",
+        seen_children_same_line = "PrintOptions::default().seen_children_same_line",
+        only_child_below = "PrintOptions::default().only_child_below",
         tensor_index_literal = "PrintOptions::default().tensor_index_literal"
     )]
     pub fn new(
@@ -203,6 +211,8 @@ impl PrintOptions {
         comment_arg_names: bool,
         commenters: Vec<CircuitCommenter>,
         sync_tensors: bool,
+        seen_children_same_line: bool,
+        only_child_below: bool,
         tensor_index_literal: bool,
     ) -> Result<Self> {
         let result = Self {
@@ -217,6 +227,8 @@ impl PrintOptions {
             comment_arg_names,
             commenters,
             sync_tensors,
+            seen_children_same_line,
+            only_child_below,
             tensor_index_literal,
         };
         result.validate()?;
@@ -235,6 +247,8 @@ impl PrintOptions {
         comment_arg_names = "Default::default()",
         commenters = "Default::default()",
         sync_tensors = "Default::default()",
+        seen_children_same_line = "Default::default()",
+        only_child_below = "Default::default()",
         tensor_index_literal = "Default::default()"
     )]
     pub fn evolve(
@@ -250,6 +264,8 @@ impl PrintOptions {
         comment_arg_names: MaybeNotSet<bool>,
         commenters: MaybeNotSet<Vec<CircuitCommenter>>,
         sync_tensors: MaybeNotSet<bool>,
+        seen_children_same_line: MaybeNotSet<bool>,
+        only_child_below: MaybeNotSet<bool>,
         tensor_index_literal: MaybeNotSet<bool>,
     ) -> Result<Self> {
         let cloned = self.clone();
@@ -271,6 +287,10 @@ impl PrintOptions {
             comment_arg_names.0.unwrap_or(cloned.comment_arg_names),
             commenters.0.unwrap_or(cloned.commenters),
             sync_tensors.0.unwrap_or(cloned.sync_tensors),
+            seen_children_same_line
+                .0
+                .unwrap_or(cloned.seen_children_same_line),
+            only_child_below.0.unwrap_or(cloned.only_child_below),
             tensor_index_literal
                 .0
                 .unwrap_or(cloned.tensor_index_literal),
@@ -320,6 +340,16 @@ impl PrintOptions {
                 "bijection print cant terminate early, so traversal must be None"
             ))
         }
+        if self.bijection && self.seen_children_same_line {
+            bail!(anyhow!(
+                "bijection doesn't support seen children same line, but technically could"
+            ))
+        }
+        if self.seen_children_same_line && self.comment_arg_names {
+            bail!(anyhow!(
+                "children same line + comment arg names not supported"
+            ))
+        }
         Ok(())
     }
 
@@ -613,6 +643,73 @@ impl PrintOptions {
                 };
             write_comment(result, node_comments, &selfy.commenters, circ.crc())?;
             result.push('\n');
+            if selfy.seen_children_same_line {
+                let child_prints_seen = circ
+                    .children()
+                    .filter_map(|c| seen_hashes.get(&c.info().hash))
+                    .join(HORIZ_BAR);
+                let unseen: Vec<_> = circ
+                    .children()
+                    .zip(new_traversal_per)
+                    .filter(|(c, _t)| !seen_hashes.contains_key(&c.info().hash))
+                    .enumerate()
+                    .collect();
+                if !child_prints_seen.is_empty() {
+                    let mut spacing = last_child_arrows(
+                        &is_last_child
+                            .iter()
+                            .copied()
+                            .chain(std::iter::once(unseen.len() == 0))
+                            .collect(),
+                        true,
+                        selfy.arrows,
+                    );
+                    spacing.pop();
+                    spacing.push_str(if selfy.arrows { HORIZ_BAR } else { UP_ELBOW });
+                    write!(result, "{}{}\n", spacing, child_prints_seen).unwrap();
+                }
+                for (i, (child, new_traversal)) in &unseen {
+                    let new_last_child = is_last_child
+                        .iter()
+                        .copied()
+                        .chain(std::iter::once(*i == unseen.len() - 1))
+                        .collect();
+                    recurse(
+                        child,
+                        depth + 1,
+                        result,
+                        seen_hashes,
+                        runnning_serial_number,
+                        selfy,
+                        &new_last_child,
+                        new_traversal.clone(),
+                        disallow_name_ident,
+                        vec![],
+                    )?;
+                }
+                return Ok(());
+            }
+            if selfy.only_child_below && circ.children().count() == 1 {
+                write!(
+                    result,
+                    "{}▼\n",
+                    last_child_arrows(&is_last_child, false, selfy.arrows)
+                )
+                .unwrap();
+                recurse(
+                    &circ.children().next().unwrap(),
+                    depth,
+                    result,
+                    seen_hashes,
+                    runnning_serial_number,
+                    selfy,
+                    &is_last_child,
+                    new_traversal_per[0].clone(),
+                    disallow_name_ident,
+                    vec![],
+                )?;
+                return Ok(());
+            }
             for (i, (child, new_traversal)) in circ.children().zip(new_traversal_per).enumerate() {
                 let new_last_child = is_last_child
                     .iter()
