@@ -2,7 +2,9 @@ use std::io::prelude::*;
 
 use anyhow::{bail, Context, Result};
 use circuit_base::{
-    flat_concat, get_compatible_dtype, module::conform_all_modules, print::PrintOptions,
+    flat_concat, get_compatible_dtype,
+    module::conform_all_modules,
+    print::{print_circuit_stats, PrintOptions},
     CircuitNode, CircuitRc, ModuleSpec,
 };
 use num_bigint::BigUint;
@@ -226,7 +228,7 @@ impl OptimizationContext {
     }
 }
 
-#[pyfunction]
+#[pyfunction(settings = "Default::default()")]
 #[pyo3(name = "optimize_circuit")]
 pub fn optimize_circuit_py(circuit: CircuitRc, settings: OptimizationSettings) -> CircuitRc {
     let mut context = OptimizationContext {
@@ -245,6 +247,7 @@ pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -
     if context.settings.verbose >= 4 {
         python_println!("Original Circuit");
         circuit.printu();
+        print_circuit_stats(&*circuit);
     }
     if context.settings.save_circ {
         let s = circuit.repru();
@@ -272,6 +275,7 @@ pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -
         PrintOptions::compiler_default()
             .print(circuit.clone())
             .unwrap();
+        print_circuit_stats(&*circuit);
     }
     let circuit = timed!(
         apply_fn_until_same(&circuit, |x| {
@@ -288,6 +292,7 @@ pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -
         PrintOptions::compiler_default()
             .print(circuit.clone())
             .unwrap();
+        print_circuit_stats(&*circuit);
     }
     let circuit = timed!(deep_heuristic_nest_adds(circuit), 10, print_timings);
     let circuit = timed!(deep_canonicalize(circuit, context), 10, print_timings);
@@ -300,11 +305,12 @@ pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -
         PrintOptions::compiler_default()
             .print(circuit.clone())
             .unwrap();
+        print_circuit_stats(&*circuit);
     }
     circuit
 }
 
-#[pyfunction]
+#[pyfunction(settings = "Default::default()")]
 pub fn optimize_to_schedule(
     circuit: CircuitRc,
     settings: OptimizationSettings,
@@ -336,7 +342,7 @@ pub fn optimize_to_schedule(
     Ok(schedule)
 }
 
-#[pyfunction]
+#[pyfunction(settings = "Default::default()")]
 pub fn optimize_and_evaluate(circuit: CircuitRc, settings: OptimizationSettings) -> Result<Tensor> {
     if !circuit.info().is_explicitly_computable {
         bail!(SchedulingError::NotExplicitlyComputable { circuit });
@@ -352,7 +358,7 @@ pub fn optimize_and_evaluate(circuit: CircuitRc, settings: OptimizationSettings)
 /// by flat-concatting and then passing around as one circuit
 /// flat concatting then splitting is an extra copy on all return data,
 /// which we could get rid of by removing flat-concat after rewrites (which would only work with a black box flat_concat node bc simplification isn't guaranteed to preserve concat at at end)
-#[pyfunction]
+#[pyfunction(settings = "Default::default()")]
 pub fn optimize_and_evaluate_many(
     circuits: Vec<CircuitRc>,
     settings: OptimizationSettings,
@@ -361,7 +367,7 @@ pub fn optimize_and_evaluate_many(
     Ok(schedule.evaluate_many(settings))
 }
 
-#[pyfunction]
+#[pyfunction(settings = "Default::default()")]
 pub fn optimize_to_schedule_many(
     circuits: Vec<CircuitRc>,
     settings: OptimizationSettings,
