@@ -4,7 +4,7 @@ use circuit_base::{prelude::*, Add, Concat, Einsum, GeneralFunction, Index};
 use num_bigint::BigUint;
 use pyo3::prelude::*;
 use rr_util::{
-    tensor_util::{TensorAxisIndex, TensorIndex, USlice},
+    tensor_util::{Slice, TensorAxisIndex, TensorIndex, USlice},
     util::{cumsum, outer_product, AsOp},
 };
 use rustc_hash::FxHashMap as HashMap;
@@ -331,6 +331,66 @@ pub fn split_to_concat(circuit: CircuitRc, axis: usize, sections: Vec<usize>) ->
     Concat::new(operands, axis, None)
 }
 
+#[pyfunction]
+pub fn concat_elim_split(circuit: &Concat) -> Option<Index> {
+    let indexes = circuit
+        .nodes
+        .iter()
+        .map(|x| x.as_index().cloned())
+        .collect::<Option<Vec<Index>>>()?;
+    let child = indexes[0].node.clone();
+    let child_axis_map = indexes[0].child_axis_map()[0].clone();
+    if !indexes
+        .iter()
+        .all(|x| x.node == child && x.child_axis_map()[0] == child_axis_map)
+    {
+        return None;
+    }
+    let axis_lower = indexes[0].child_axis_map_including_slices()[0]
+        .iter()
+        .position(|i| *i == Some(circuit.axis))?;
+    let sections: Vec<Slice> = indexes
+        .iter()
+        .map(|x| match x.index.0[axis_lower] {
+            TensorAxisIndex::Slice(slice) => Some(slice),
+            _ => None,
+        })
+        .collect::<Option<Vec<Slice>>>()?;
+    let sections: Vec<USlice> = sections
+        .iter()
+        .map(|x| x.to_uslice(child.info().shape[axis_lower]))
+        .collect();
+    let mut place = 0;
+    for s in &sections {
+        if s.start != place {
+            return None;
+        }
+        place = s.stop;
+    }
+    let new_axis_index = TensorAxisIndex::Slice(Slice {
+        start: Some(sections[0].start as i64),
+        stop: Some(place as i64),
+    });
+    let mut new_index = vec![];
+    for i in 0..child.info().rank() {
+        if i == axis_lower {
+            new_index.push(new_axis_index.clone());
+        } else {
+            for (j, idx) in indexes.iter().enumerate() {
+                if idx.index.0[i] != indexes[0].index.0[i] {
+                    return None;
+                }
+            }
+            new_index.push(indexes[0].index.0[i].clone());
+        }
+    }
+    Some(Index::new(
+        child,
+        TensorIndex(new_index),
+        circuit.name_cloned(),
+    ))
+}
+
 pub fn split_sections(l: usize, sections: usize) -> Vec<usize> {
     let even_size = l / sections;
     let residual = l % sections;
