@@ -1,3 +1,8 @@
+use std::{
+    iter::zip,
+    simd::{u64x8, u8x8, SimdPartialEq, SimdUint},
+};
+
 use anyhow::{bail, Result};
 use circuit_base::{
     deep_map_op_context, deep_map_preorder_unwrap, deep_map_unwrap, on_circuit_names, prelude::*,
@@ -11,7 +16,7 @@ use rr_util::{
     python_error_exception, timed, timed_value,
     util::{apply_fn_until_none, apply_fn_until_same, mapping_until_end, AsOp, HashBytes},
 };
-use rustc_hash::FxHashMap as HashMap;
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use thiserror::Error;
 
 use crate::{
@@ -20,7 +25,7 @@ use crate::{
     canonicalize::deep_canonicalize,
     circuit_optimizer::OptimizationContext,
     concat_rewrite::{
-        add_pull_concat, concat_drop_size_zero, concat_fuse, einsum_pull_concat,
+        add_pull_concat, concat_drop_size_zero, concat_elim_split, concat_fuse, einsum_pull_concat,
         generalfunction_pull_concat, index_concat_drop_unreached,
     },
     diag_rewrite::{add_pull_diags, einsum_push_down_trace},
@@ -183,6 +188,7 @@ fn all_simp_fns() -> SimpFns {
         ],
         concat: vec![
             l_wrap!(concat_elim_identity),
+            f_wrap!(concat_elim_split),
             l_wrap!(concat_pull_removable_axes),
             l_wrap!(concat_merge_uniform),
             f_wrap!(concat_drop_size_zero),
@@ -550,24 +556,38 @@ pub fn deep_push_down_index_raw(circ: CircuitRc, min_size: Option<usize>) -> Cir
 
 #[derive(Debug, Clone, Default)]
 pub struct CircBloomFilter {
-    pub bits: u128,
+    pub bitarrays: u64x8,
+    /* based on hyperparam search, 64x8 seems good for cases where there are a lot of true positives
+     * and 7M total tests
+     * note: more than 8 requires other input format, currently its 8 bytes which each go to a bitmap
+     */
 }
 impl CircBloomFilter {
     #[inline]
-    pub fn insert(&mut self, x: usize) {
-        self.bits |= 1 << (x & 0b1111111)
+    fn input_to_bits(x: u64) -> u64x8 {
+        let mut inp_bytes: u8x8 = x.to_ne_bytes().into();
+        inp_bytes &= u8x8::from([0b111111; 8]);
+        let mut shifty: u64x8 = [1; 8].into();
+        shifty <<= inp_bytes.cast::<u64>();
+        shifty
     }
     #[inline]
-    pub fn contains(&mut self, x: usize) -> bool {
-        self.bits & 1 << (x & 0b1111111) != 0
+    pub fn insert(&mut self, x: u64) {
+        let shifty = Self::input_to_bits(x);
+        self.bitarrays |= shifty;
+    }
+    #[inline]
+    pub fn contains(&mut self, x: u64) -> bool {
+        let shifty = Self::input_to_bits(x);
+        !(self.bitarrays & shifty).simd_eq(u64x8::from([0; 8])).any()
     }
     #[inline]
     pub fn is_superset(&self, other: &CircBloomFilter) -> bool {
-        other.bits & !self.bits == 0
+        other.bitarrays & !self.bitarrays == u64x8::default()
     }
     #[inline]
     pub fn intersection_count(&self, other: &CircBloomFilter) -> u32 {
-        (other.bits & self.bits).count_ones()
+        (other.bitarrays[0] & self.bitarrays[0]).count_ones() // not well founded like other bloom filter ops
     }
 }
 
@@ -575,10 +595,11 @@ impl CircBloomFilter {
 /// in future adds
 /// this is suboptimal in many ways. one is broadcasts allow outer products which should be avoided but aren't
 /// for each add, greedily nest into preexisting adds
+#[inline]
 fn child_bloom_filter(add: &Add) -> CircBloomFilter {
     let mut result: CircBloomFilter = Default::default();
     for node in &add.nodes {
-        result.insert(node.info().hash_usize());
+        result.insert(node.info().hash_usize() as u64);
     }
     result
 }
@@ -591,8 +612,8 @@ pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
             seen_adds.push((add.clone(), child_bloom_filter(add)));
         }
     });
-    // TODO: profile and optimize these
-
+    // let mut stats = (0, 0, 0);
+    // let mut add_width_tracker: Vec<usize> = vec![0; 300];
     let mut intersections: HashMap<Add, CircBloomFilter> = HashMap::default();
     for circ in &seen_adds {
         for circ2 in &seen_adds {
@@ -621,25 +642,32 @@ pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
         .enumerate()
         .map(|(i, (c, _b))| (c.info().hash, i))
         .collect();
+    // for ad in &seen_adds {
+    //     add_width_tracker[ad.0.nodes.len()] += 1;
+    // }
     let mut mapping: HashMap<Add, Add> = HashMap::default();
-    for _ in 0..3 {
+    for _ in 0..6 {
         let mut to_do: Vec<((Add, CircBloomFilter), (Add, CircBloomFilter))> = vec![];
         for cand_sup in &seen_adds {
             if cand_sup.0.nodes.len() >= 3 {
                 for cand_sub in &seen_adds {
+                    // stats.0 += 1;
                     if cand_sup.1.is_superset(&cand_sub.1)
                         && cand_sub.0.nodes.len() < cand_sup.0.nodes.len()
                         && cand_sub.0 != cand_sup.0
                     {
+                        // stats.1 += 1;
                         if let Some(new) = extract_add(&cand_sup.0, &cand_sub.0) {
+                            // stats.2 += 1;
                             let new_entry = (new.clone(), child_bloom_filter(&new));
                             to_do.push(((cand_sup.0.clone(), cand_sup.1.clone()), new_entry));
+                            break;
                         }
                     }
                 }
             }
         }
-        // println!("nest op counts bloom {} extract {} join {}",bloom_count,extract_count,join_count);
+        // println!("stats {:?} {:?}", stats, add_width_tracker);
         if to_do.is_empty() {
             break;
         }
@@ -650,6 +678,7 @@ pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
                 seen_adds[si] = to_insert;
             } else {
                 seen_index = Some(seen_adds.len());
+                // add_width_tracker[to_insert.0.nodes.len()] += 1;
                 seen_adds.push(to_insert);
             }
             if !seen_adds_index.contains_key(&new_sup.0.info().hash) {
