@@ -1,7 +1,7 @@
 use circuit_base::{
-    evaluate_fn_dtype_device,
+    evaluate_device_dtype,
     generalfunction::{SpecTrait, OFFICIAL_GENERALFUNCTION_INVERSES},
-    Circuit, CircuitNode, CircuitRc, GeneralFunction, ScalarConstant,
+    Circuit, CircuitNode, CircuitRc, GeneralFunction, Scalar,
 };
 use pyo3::prelude::*;
 use rr_util::{sv, tensor_util::TorchDeviceDtypeOp};
@@ -20,10 +20,10 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
         let name_str: &str = &circuit.spec.name();
         match name_str {
             "softmax" => {
-                if let Circuit::ScalarConstant(_sc) = &**circuit.nodes[0] {
+                if let Circuit::Scalar(_sc) = &**circuit.nodes[0] {
                     let scalar: f64 = 1.0
                         / circuit.nodes[0].info().shape[circuit.nodes[0].info().rank() - 1] as f64;
-                    return Some(ScalarConstant::nrc(
+                    return Some(Scalar::nrc(
                         scalar,
                         circuit.info().shape.clone(),
                         circuit.name_cloned(),
@@ -31,11 +31,11 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
                 }
             }
             "log_softmax" => {
-                if let Circuit::ScalarConstant(_sc) = &**circuit.nodes[0] {
+                if let Circuit::Scalar(_sc) = &**circuit.nodes[0] {
                     let scalar: f64 = (1.0
                         / circuit.nodes[0].info().shape[circuit.nodes[0].info().rank() - 1] as f64)
                         .ln();
-                    return Some(ScalarConstant::nrc(
+                    return Some(Scalar::nrc(
                         scalar,
                         circuit.info().shape.clone(),
                         circuit.name_cloned(),
@@ -56,10 +56,10 @@ pub fn generalfunction_evaluate_simple(node: &GeneralFunction) -> Option<Circuit
         && node.nodes.len() == 1
         && node.nodes[0].info().rank() == 0
     {
-        return node.nodes[0].as_scalar_constant().map(|_inner| {
+        return node.nodes[0].as_scalar().map(|_inner| {
             Python::with_gil(|py| {
-                ScalarConstant::new(
-                    evaluate_fn_dtype_device(
+                Scalar::new(
+                    evaluate_device_dtype(
                         node.crc(),
                         TorchDeviceDtypeOp {
                             device: None,
