@@ -155,20 +155,26 @@ impl Dag {
         // );
         // self.print_cytoscape_link();
         let mut result = false;
-        while self.simplify_pass(simp_settings) {
+        while self.simplify_pass(simp_settings, false) {
+            if simp_settings.verbose > 2 {
+                println!("did pass {}", self.node_to_orig.len());
+                self.print_cytoscape_link();
+            }
             result = true;
         }
-        self.simplify_pass(simp_settings);
-        // println!(
-        //     "SIMPLIFIED to_orig {:?} children {:?} parents {:?} pre {:?}",
-        //     &self.node_to_orig, &self.children, &self.parents, &self.pre_root_nodes
-        // );
-        // self.print_cytoscape_link();
+        while self.simplify_pass(simp_settings, true) {
+            self.print_cytoscape_link();
+            if simp_settings.verbose > 2 {
+                println!("did pass {}", self.node_to_orig.len());
+            }
+            result = true;
+        }
+        self.simplify_pass(simp_settings, true);
         result
     }
 
     // warning: I'm modifying the graph as I'm traversing it. edits must not mess with topo frontier
-    pub fn simplify_pass(&mut self, simp_settings: &DagSimpSettings) -> bool {
+    pub fn simplify_pass(&mut self, simp_settings: &DagSimpSettings, do_isolated: bool) -> bool {
         // simplify bottom_up
         let mut did_anything = false;
 
@@ -200,11 +206,13 @@ impl Dag {
                 and_namer!(Dag::try_elim_small_leaf),
                 and_namer!(Dag::try_merge_descending_peaks),
             ];
-            for (_simp_name, simplification) in simps {
+            for (simp_name, simplification) in simps {
                 if let Some(the_parenties) = simplification(self, popped, simp_settings) {
                     parenties = the_parenties;
                     did_anything = true;
-                    // println!("did simp {}", simp_name);
+                    if simp_settings.verbose > 2 {
+                        println!("did simp {}", simp_name);
+                    }
                     break;
                 }
             }
@@ -215,7 +223,9 @@ impl Dag {
                         .iter()
                         .all(|x| nodes_visited.contains(x))
                     {
-                        bottom_frontier.push(*parent);
+                        if !nodes_visited.contains(parent) {
+                            bottom_frontier.push(*parent);
+                        }
                     }
                 }
             } else {
@@ -224,7 +234,9 @@ impl Dag {
                         .iter()
                         .all(|x| nodes_visited.contains(x))
                     {
-                        bottom_frontier.push(*parent);
+                        if !nodes_visited.contains(parent) {
+                            bottom_frontier.push(*parent);
+                        }
                     }
                 }
             }
@@ -593,6 +605,9 @@ impl Dag {
             };
             // assert!(order[0].0==iso.1 && order.last().unwrap().0==iso.0);
             self.replace_isolated_with_linear(&order);
+            if settings.verbose > 2 {
+                println!("did isolated");
+            }
             // println!("FINISHED ISOLATED SCHEDULE");
         }
         // self.print_cytoscape_link();
@@ -893,7 +908,7 @@ impl Dag {
                 .join(",")
         );
         python_println!(
-            "http://104.171.200.179:4573/#/networkgraph/{}",
+            "http://interp-tools.redwoodresearch.org/#/networkgraph/{}",
             url_escape::encode_fragment(&not_excaped)
         )
     }
