@@ -140,6 +140,18 @@ impl RegexWrap {
         self.regex.is_match(s)
     }
 
+    pub fn __repr__(&self) -> String {
+        let escape_dot_str = if self.escape_dot { "True" } else { "False" };
+        format!(
+            "Regex(\"{}\", escape_dot={})",
+            self.regex
+                .to_string()
+                .replace('\\', r"\\")
+                .replace('"', "\\\""),
+            escape_dot_str
+        )
+    }
+
     #[getter]
     pub fn pattern(&self) -> &str {
         &self.pattern
