@@ -1,5 +1,6 @@
 use std::{
     collections::{HashMap, HashSet},
+    fmt::Debug,
     iter::zip,
     sync::{Arc, Mutex},
 };
@@ -16,11 +17,11 @@ use itertools::Itertools;
 use macro_rules_attribute::apply;
 use pyo3::{exceptions::PyValueError, prelude::*};
 use rr_util::{
-    cached_method,
+    cached_method, fn_struct,
     py_types::{PyShape, Tensor, PY_UTILS},
     python_error_exception,
     tensor_util::{Shape, Slice, TensorAxisIndex, TensorIndex},
-    util::HashBytes,
+    util::{flip_op_result, HashBytes},
 };
 use smallvec::SmallVec as Sv;
 use thiserror::Error;
@@ -30,6 +31,8 @@ use crate::{
     Matcher, Transform, TransformRc,
 };
 
+fn_struct!(pub Seeder:Fn(circuit:CircuitRc)->Option<i32>);
+
 fn count_elements(s: &Shape) -> usize {
     s.iter().product()
 }
@@ -95,6 +98,8 @@ pub struct RandomSampleSpec {
     replace: bool,
     #[pyo3(get, set)]
     probs_and_group_evaluation_settings: OptimizationSettings,
+    #[pyo3(get, set)]
+    seeder: Option<Seeder>,
     // kinda dumb that we use a mutex here. Certainly overkill in a particular sense...
     indices_cache: Arc<Mutex<cached::UnboundCache<HashBytes, Tensor>>>,
 }
@@ -166,17 +171,20 @@ impl RandomSampleSpec {
     #[args(
         shape = "RandomSampleSpec::default().shape",
         replace = "RandomSampleSpec::default().replace",
-        probs_and_group_evaluation_settings = "RandomSampleSpec::default().probs_and_group_evaluation_settings"
+        probs_and_group_evaluation_settings = "RandomSampleSpec::default().probs_and_group_evaluation_settings",
+        seeder = "None"
     )]
     fn new(
         shape: Shape,
         replace: bool,
         probs_and_group_evaluation_settings: OptimizationSettings,
+        seeder: Option<Seeder>,
     ) -> PyClassInitializer<Self> {
         Self {
             shape,
             replace,
             probs_and_group_evaluation_settings,
+            seeder,
             ..Default::default()
         }
         .into_init()
@@ -194,10 +202,20 @@ impl RandomSampleSpec {
             return Ok(out.clone());
         }
 
-        let prob_tensor =
-            optimize_and_evaluate(probs_and_group, self.probs_and_group_evaluation_settings)
-                .context("failed to evaluate prob group")?;
-        let out = sample_indices(prob_tensor, self.shape.clone(), self.replace)
+        let prob_tensor = optimize_and_evaluate(
+            probs_and_group.clone(),
+            self.probs_and_group_evaluation_settings,
+        )
+        .context("failed to evaluate prob group")?;
+
+        let seed = flip_op_result(
+            self.seeder
+                .clone()
+                .map(|seed_fn| seed_fn.call(probs_and_group)),
+        )?
+        .flatten();
+
+        let out = sample_indices(prob_tensor, self.shape.clone(), self.replace, seed)
             .context("failed to sample indices for discrete var")?;
 
         self.indices_cache
@@ -214,16 +232,22 @@ impl Default for RandomSampleSpec {
             shape: Sv::new(),
             replace: true,
             probs_and_group_evaluation_settings: Default::default(),
+            seeder: None,
             indices_cache: Arc::new(Mutex::new(cached::UnboundCache::new())),
         }
     }
 }
 
-fn sample_indices(input: Tensor, shape: Shape, replacement: bool) -> Result<Tensor, PyErr> {
+fn sample_indices(
+    input: Tensor,
+    shape: Shape,
+    replacement: bool,
+    seed: Option<i32>,
+) -> Result<Tensor, PyErr> {
     let indices = Python::with_gil(|py| {
         PY_UTILS
             .random_indices
-            .call(py, (input, shape.clone(), replacement), None)?
+            .call(py, (input, shape.clone(), replacement, seed), None)?
             .extract::<Tensor>(py)
     })?;
     assert_eq!(indices.shape(), &shape);
