@@ -157,8 +157,10 @@ pub fn save_tensor(tensor: Tensor, force: bool) -> Result<bool> {
     fs::create_dir_all(&cache_dir)?;
 
     let result = write_tensor_to_dir_tree(&cache_dir, tensor.clone(), force);
-    if let Ok(did_write) = result && did_write{
-        register_tensor_unsynced(&tensor.hashed().hash_base16().unwrap())?
+    if force{
+        write_tensor_to_dir_tree(&get_rrfs_tensor_db_dir(), tensor.clone(), force)?;
+    }else if let Ok(did_write) = result && did_write{
+        register_tensor_unsynced(&tensor.hashed().hash_base16().unwrap())?;
     }
     result
 }
@@ -194,7 +196,6 @@ pub fn sync_all_unsynced_tensors(parallelism: usize) -> Result<()> {
         pool.execute(move || {
             tx.send((move || {
                 let local_filename = get_filename_dir_tree(&get_tensor_cache_dir(), &line)?;
-                println!("have dir tree");
                 let remote_filename =
                     local_filename.replace(&get_tensor_cache_dir(), &get_rrfs_tensor_db_dir());
                 let remote_filename_dir: Vec<_> = remote_filename.split("/").collect();
@@ -204,7 +205,7 @@ pub fn sync_all_unsynced_tensors(parallelism: usize) -> Result<()> {
                 fs::create_dir_all(&remote_filename_dir).context("failed to create dir in tree")?;
                 copy_without_setting_permissions(&local_filename, &remote_filename)
                     .context("failed to copy")?;
-                print!("wrote");
+                eprintln!("wrote {}", line);
                 Ok::<(), anyhow::Error>(())
             })())
             .unwrap()
