@@ -31,6 +31,14 @@ impl IntoPy<PyObject> for HashBytesToPy {
     }
 }
 
+pub fn hash_to_string(hash: &HashBytes) -> String {
+    hash.iter()
+        .map(|b| {
+            alphabet()[((*b) % 52) as usize].clone() + &alphabet()[((*b) / 52) as usize].clone()
+        })
+        .join("")
+}
+
 /// letters used in einsum and rearrange strings
 /// i wanted this to be a constant, but that is annoying in rust bc can't do any computation to produce constants
 /// so instead it's cached function
@@ -658,3 +666,20 @@ pub fn indent(x: String, width: usize) -> String {
 
     x.lines().map(|s| tab.clone() + s).join("\n")
 }
+
+/// https://stackoverflow.com/questions/64498617/how-to-transpose-a-vector-of-vectors-in-rust
+pub fn transpose<T>(v: Vec<Vec<T>>, inner_len: usize) -> Vec<Vec<T>> {
+    for x in &v {
+        assert_eq!(x.len(), inner_len);
+    }
+
+    let mut iters: Vec<_> = v.into_iter().map(|n| n.into_iter()).collect();
+    (0..inner_len)
+        .map(|_| {
+            iters
+                .iter_mut()
+                .map(|n| n.next().unwrap())
+                .collect::<Vec<T>>()
+        })
+        .collect()
+}
